@@ -17,6 +17,7 @@ from datetime import datetime
 from numpy.ma import MaskedArray
 import typing
 
+from roguewave.metoceandata import WaveBulkData
 
 class WaveSpectrumInput(TypedDict):
     frequency: List[float]
@@ -26,43 +27,6 @@ class WaveSpectrumInput(TypedDict):
     longitude: Union[float, None]
 
 
-class BulkVariables():
-    def __init__(self, spectrum):
-        if spectrum:
-            self.m0 = spectrum.m0()
-            self.hm0 = spectrum.hm0()
-            self.tm01 = spectrum.tm01()
-            self.tm02 = spectrum.tm02()
-            self.peak_period = spectrum.peak_period()
-            self.peak_direction = spectrum.peak_direction()
-            self.peak_spread = spectrum.peak_spread()
-            self.bulk_direction = spectrum.bulk_direction()
-            self.bulk_spread = spectrum.bulk_spread()
-            self.peak_frequency = spectrum.peak_frequency()
-            self.peak_wavenumber = spectrum.peak_wavenumber()
-            self.timestamp = spectrum.timestamp
-            self.latitude = spectrum.latitude
-            self.longitude = spectrum.longitude
-        else:
-            self._nanify()
-            self.timestamp = numpy.nan
-            self.latitude = numpy.nan
-            self.longitude = numpy.nan
-
-    def _nanify(self):
-        self.m0 = numpy.nan
-        self.hm0 = numpy.nan
-        self.tm01 = numpy.nan
-        self.tm02 = numpy.nan
-        self.peak_period = numpy.nan
-        self.peak_direction = numpy.nan
-        self.peak_spread = numpy.nan
-        self.bulk_direction = numpy.nan
-        self.bulk_spread = numpy.nan
-        self.peak_frequency = numpy.nan
-        self.peak_wavenumber = numpy.nan
-
-
 class WaveSpectrum():
     """
     Base spectral class.
@@ -73,7 +37,8 @@ class WaveSpectrum():
     angular_convention = 'Wave travel direction (going-to), measured anti-clockwise from East'
     bulk_properties = (
         'm0', 'hm0', 'tm01', 'tm02', 'peak_period', 'peak_direction',
-        'peak_spread','bulk_direction', 'bulk_spread', 'peak_frequency',
+        'peak_directional_spread', 'mean_direction', 'mean_directional_spread',
+        'peak_frequency',
         'peak_wavenumber', 'latitude', 'longitude', 'timestamp')
 
     def __init__(self,
@@ -89,26 +54,27 @@ class WaveSpectrum():
         self._peak_wavenumber = None
 
         # Type conversions are needed because the JSON serializer does not accept float32
-        self.frequency = numpy.array(wave_spectrum_input['frequency'],dtype='float64')
-        density = numpy.array(wave_spectrum_input['varianceDensity'],dtype='float64')
+        self.frequency = numpy.array(wave_spectrum_input['frequency'],
+                                     dtype='float64')
+        density = numpy.array(wave_spectrum_input['varianceDensity'],
+                              dtype='float64')
         mask = (density < 0) | numpy.isnan(density)
         density[mask] = numpy.nan
 
         self._variance_density = MaskedArray(
-            density,dtype='float64',mask=mask)
+            density, dtype='float64', mask=mask)
         self.timestamp = to_datetime(wave_spectrum_input['timestamp'])
 
         # There are cases that wavefleet returns None for latitude or longitude.
         # This is a bug - but to avoid issues we catch it here.
         if (wave_spectrum_input['longitude'] is None) or \
-            (wave_spectrum_input['latitude']) is None:
+                (wave_spectrum_input['latitude']) is None:
             self.longitude = None
             self.latitude = None
         else:
             self.longitude = float(wave_spectrum_input['longitude'])
             self.latitude = float(wave_spectrum_input['latitude'])
 
-
     def frequency_moment(self, power: int, fmin=0, fmax=numpy.inf) -> float:
         pass
 
@@ -199,7 +165,7 @@ class WaveSpectrum():
         return numpy.sqrt(self.m0(fmin, fmax) / self.m2(fmin, fmax))
 
     def peak_index(self, fmin=0, fmax=numpy.inf) -> float:
-        if fmin==0 and fmax==numpy.inf:
+        if fmin == 0 and fmax == numpy.inf:
             if self._peak_index is None:
                 self._peak_index = numpy.argmax(self.e)
             return self._peak_index
@@ -221,7 +187,7 @@ class WaveSpectrum():
         b1 = self.b1[index]
         return self._mean_direction(a1, b1)
 
-    def peak_spread(self, fmin=0, fmax=numpy.inf):
+    def peak_directional_spread(self, fmin=0, fmax=numpy.inf):
         index = self.peak_index(fmin, fmax)
         a1 = self.a1[index]
         b1 = self.b1[index]
@@ -237,11 +203,11 @@ class WaveSpectrum():
             2 - 2 * numpy.sqrt(a1 ** 2 + b1 ** 2)) * 180 / numpy.pi
 
     @property
-    def mean_direction(self):
+    def mean_direction_per_frequency(self):
         return self._mean_direction(self.a1, self.b1)
 
     @property
-    def mean_spread(self):
+    def mean_spread_per_frequency(self):
         return self._spread(self.a1, self.b1)
 
     def _spectral_weighted(self, property, fmin=0, fmax=numpy.inf):
@@ -251,23 +217,23 @@ class WaveSpectrum():
         return numpy.trapz(property[range] * self.e[range],
                            self.frequency[range]) / self.m0(fmin, fmax)
 
-    def bulk_direction(self, fmin=0, fmax=numpy.inf):
-        return self._mean_direction(self.bulk_a1(fmin, fmax),
-                                    self.bulk_b1(fmin, fmax))
+    def mean_direction(self, fmin=0, fmax=numpy.inf):
+        return self._mean_direction(self.mean_a1(fmin, fmax),
+                                    self.mean_b1(fmin, fmax))
 
-    def bulk_spread(self, fmin=0, fmax=numpy.inf):
-        return self._spread(self.bulk_a1(fmin, fmax), self.bulk_b1(fmin, fmax))
+    def mean_directional_spread(self, fmin=0, fmax=numpy.inf):
+        return self._spread(self.mean_a1(fmin, fmax), self.mean_b1(fmin, fmax))
 
-    def bulk_a1(self, fmin=0, fmax=numpy.inf):
+    def mean_a1(self, fmin=0, fmax=numpy.inf):
         return self._spectral_weighted(self.a1, fmin, fmax)
 
-    def bulk_b1(self, fmin=0, fmax=numpy.inf):
+    def mean_b1(self, fmin=0, fmax=numpy.inf):
         return self._spectral_weighted(self.b1, fmin, fmax)
 
-    def bulk_a2(self, fmin=0, fmax=numpy.inf):
+    def mean_a2(self, fmin=0, fmax=numpy.inf):
         return self._spectral_weighted(self.a2, fmin, fmax)
 
-    def bulk_b2(self, fmin=0, fmax=numpy.inf):
+    def mean_b2(self, fmin=0, fmax=numpy.inf):
         return self._spectral_weighted(self.b2, fmin, fmax)
 
     def U10(self, **kwargs) -> Tuple[float, float]:
@@ -317,8 +283,20 @@ class WaveSpectrum():
             ustar = self.Ustar()
         return phase_velocity(self.wavenumber(depth), depth) / ustar
 
-    def bulk_variables(self) -> BulkVariables:
-        return BulkVariables(self)
+    def bulk_variables(self) -> WaveBulkData:
+        return WaveBulkData(
+            significant_waveheight=self.significant_waveheight,
+            mean_period=self.mean_period,
+            peak_period=self.peak_period(),
+            peak_direction=self.peak_direction(),
+            peak_directional_spread=self.peak_directional_spread(),
+            mean_direction=self.mean_direction(),
+            mean_directional_spread=self.mean_directional_spread(),
+            peak_frequency=self.peak_frequency(),
+            timestamp=self.timestamp,
+            latitude=self.latitude,
+            longitude=self.longitude
+        )
 
     def copy(self):
         pass
@@ -326,11 +304,18 @@ class WaveSpectrum():
     def __add__(self, other) -> "WaveSpectrum":
         pass
 
+    @property
+    def significant_waveheight(self):
+        return self.hm0()
+
+    @property
+    def mean_period(self):
+        return self.tm01()
+
 
 def extract_bulk_parameter(parameter, spectra: typing.List[WaveSpectrum]):
-
     if parameter == 'timestamp':
-        output = numpy.empty(len(spectra),dtype='object')
+        output = numpy.empty(len(spectra), dtype='object')
     else:
         output = numpy.empty(len(spectra))
 
@@ -340,4 +325,5 @@ def extract_bulk_parameter(parameter, spectra: typing.List[WaveSpectrum]):
             output[index] = temp()
         else:
             output[index] = temp
-    return output
\ No newline at end of file
+    return output
+
